from ehelply_microservice_library.service_bootstrap import ServiceBootstrap

from fastapi.testclient import TestClient

from typing import Type, List

import json
import os
from pathlib import Path
from pydantic import BaseModel


class TestHeaders(BaseModel):
    access_token: str
    secret_token: str
    project_uuid: str

    def to_client_headers(self) -> dict:
        return {
            "ehelply-project": self.project_uuid,
            "x-access-token": self.access_token,
            "x-secret-token": self.secret_token
        }


def make_test_client(service: Type[ServiceBootstrap], additional_facts: List[str] = []) -> TestClient:
    class ServiceTest(service):
        def if_dev_launch_dev_server(self) -> bool:
            return False

        def get_fact_subscriptions(self) -> List[str]:
            return super().get_fact_subscriptions() + additional_facts

    service = ServiceTest()

    app = service.fastapi_driver.instance

    client = TestClient(app)

    return client


def make_test_headers() -> TestHeaders:
    root_path = Path(os.getcwd())
    credentials_file = Path(root_path).resolve().joinpath('credentials.testing.json')

    with open(str(credentials_file)) as file:
        return TestHeaders(**json.load(file))

    raise Exception("Invalid credentials file")



from ehelply_bootstrapper.integrations.integration import Integration
import threading
from ehelply_logger.Logger import Logger

import requests
import time

from typing import List

from pydantic import BaseModel
from ehelply_bootstrapper.utils.environment import Environment

from ehelply_bootstrapper.utils.state import State


class FactBase(BaseModel):
    name: str
    data: dict


def get_fact(fact_name: str):
    from ehelply_bootstrapper.utils.state import State
    fact: Fact = State.integrations.get("fact")
    return fact.facts[fact_name]


def get_fact_stage(fact_name: str):
    from ehelply_bootstrapper.utils.state import State
    fact: Fact = State.integrations.get("fact")
    try:
        return fact.facts[fact_name][Environment.stage()]
    except:
        raise Exception("Fact `" + fact_name + "` has no stage `" + Environment.stage() + "`")


def get_fact_endpoint(fact_name: str):
    from ehelply_bootstrapper.utils.state import State
    fact: Fact = State.integrations.get("fact")
    try:
        return fact.facts[fact_name][Environment.stage()]['endpoint']
    except:
        raise Exception("Fact `" + fact_name + "` has no endpoint for stage `" + Environment.stage() + "`")


class Fact(Integration):
    def __init__(self, logger: Logger, fact_names: List[str] = None) -> None:
        super().__init__("fact")
        self.fact_names = []
        if fact_names:
            self.fact_names = fact_names
        self.facts = {}
        self.logger = logger.spinoff()
        self.thread: FactThread = None
        self.fact_query_delay_minutes: int = 20

        # self.m2m = State.integrations.get("m2m")

    def init(self):
        # Initial load to prevent race conditions
        self._query()
        self.thread = FactThread(self).start()

    def load(self):
        pass

    def _query(self):
        State.logger.info("  -> Loading Facts")
        for fact_name in self.fact_names:
            State.logger.info("     * " + State.config.facts.endpoint + "/facts/" + fact_name)
            response = requests.get(State.config.facts.endpoint + "/facts/" + fact_name)
            try:
                self.facts[fact_name] = response.json()['data']
            except:
                raise Exception(
                    "Communication with ehelply-facts failed. There are several possible reasons for this: incorrect fact name or you're sending the request from an IP address that has not been whitelisted.")

    def refresh(self):
        self._query()

    def run(self):
        while True:
            time.sleep(self.fact_query_delay_minutes * 60)
            self._query()

    def push_fact(self, fact: FactBase):
        response = requests.post(State.config.facts.endpoint + "/facts", json={"fact": fact.dict()})
        return response


class FactThread(threading.Thread):
    def __init__(self, fact: Fact):
        super().__init__()
        self.fact: Fact = fact
        self.daemon = True

    def run(self) -> None:
        self.fact.run()



from ehelply_bootstrapper.integrations.integration import Integration
from ehelply_bootstrapper.utils.state import State
from ehelply_microservice_library.integrations.fact import get_fact_endpoint
import json
from fastapi.encoders import jsonable_encoder


class Log(Integration):
    def __init__(self, service: str) -> None:
        super().__init__("log")
        self.sqs = None
        self.service = service

        self.m2m = State.integrations.get("m2m")

    def init(self):
        pass

    def load(self):
        pass

    def post_load(self):
        self.sqs = State.aws.make_client("sqs")

    def _send(self, payload: dict):
        if type(payload['log']['data']) is not dict or type(payload['log']['message']) is not str or type(
                payload['log']['subject']) is not str:
            State.logger.warning('Log entry discarded due to invalid payload.')
            return False

        queue_url = get_fact_endpoint("mq-logging")
        self.sqs.send_message(
            QueueUrl=queue_url,
            MessageBody=json.dumps(jsonable_encoder(payload))
        )

    def info(self, message: str, subject: str = "", data: dict = None):
        if not data:
            data = {}

        self._send({
            "log": {
                "message": message,
                "severity": "info",
                "subject": subject,
                "data": data
            },
            "service": self.service
        })

    def warning(self, message: str, subject: str = "", data: dict = None):
        if not data:
            data = {}

        self._send({
            "log": {
                "message": message,
                "severity": "warning",
                "subject": subject,
                "data": data
            },
            "service": self.service
        })

    def severe(self, message: str, subject: str = "", data: dict = None):
        if not data:
            data = {}

        self._send({
            "log": {
                "message": message,
                "severity": "severe",
                "subject": subject,
                "data": data
            },
            "service": self.service
        })

    def debug(self, message: str, subject: str = "", data: dict = None):
        if not data:
            data = {}

        self._send({
            "log": {
                "message": message,
                "severity": "debug",
                "subject": subject,
                "data": data
            },
            "service": self.service
        })

    def debugg(self, message: str, subject: str = "", data: dict = None):
        if not data:
            data = {}

        self._send({
            "log": {
                "message": message,
                "severity": "debugg",
                "subject": subject,
                "data": data
            },
            "service": self.service
        })

    def debuggg(self, message: str, subject: str = "", data: dict = None):
        if not data:
            data = {}

        self._send({
            "log": {
                "message": message,
                "severity": "debuggg",
                "subject": subject,
                "data": data
            },
            "service": self.service
        })



import traceback

from typing import Union, Optional

from pydantic import BaseModel

from fastapi import HTTPException

from ehelply_python_sdk.services.access.auth_rules import AuthRule, AuthException
from ehelply_python_sdk.services.access.sdk import AuthModel

from ehelply_microservice_library.integrations.monitor import Monitor


class CloudParticipant(BaseModel):
    active_participant: str
    project_uuid: str
    is_privileged: bool = False
    entity_identifier: Optional[str] = None


async def ehelply_cloud_access(
        auth: AuthModel,
        node: str,
        service_target: str,
        monitor: Monitor,
        ignore_project_enabled=False,
        ignore_spend_limits=False,
        exception_if_unauthorized=True,
        exception_if_spend_maxed=True,
        exception_if_project_not_enabled=True,
) -> Union[bool, CloudParticipant]:
    """

    Args:
        auth:
        node:
        service_target:
        monitor: Send in the monitor integration and project max spend will be checked automatically
        exception_if_unauthorized:
        exception_if_spend_maxed:

    Returns:

    """
    # Admin or M2M access
    try:
        await AuthRule(
            auth,
            exception_if_unauthorized=True
        ).participant_has_node_on_target(
            node=node,
            target_identifier=service_target,
            partition="ehelply-resources"
        ).verify()

        return CloudParticipant(
            is_privileged=True,
            active_participant=auth.active_participant_uuid,
            project_uuid=auth.project_uuid,
            entity_identifier=auth.entity_identifier
        )
    except (AuthException, HTTPException):
        pass
    except Exception as e:
        traceback.print_exc()
        raise HTTPException(
            status_code=500,
            detail="Ruh Roh, something has gone terribly wrong - Denied by eHelply."
        )

    spend_maxed: bool = False
    project_not_enabled: bool = False

    # Regular eHelply Cloud access
    try:
        await AuthRule(
            auth,
            exception_if_unauthorized=True
        ).participant_has_node_on_target(
            node=node,
            target_identifier=auth.project_uuid,
            partition="ehelply-cloud"
        ).customentity_has_node_on_target(
            node=node,
            target_identifier=service_target,
            partition="ehelply-cloud",
            entity_identifier=auth.project_uuid
        ).verify()

        project = await monitor.get_project(project_uuid=auth.project_uuid)

        if not ignore_project_enabled and project.status != "enabled":
            project_not_enabled = True
            raise AuthException("Project is not enabled")

        if not ignore_spend_limits and project.is_spend_maxed:
            spend_maxed = True
            raise AuthException("Spend is maxed")

        return CloudParticipant(
            is_privileged=False,
            active_participant=auth.active_participant_uuid,
            project_uuid=auth.project_uuid,
            entity_identifier=auth.entity_identifier
        )
    except (AuthException, HTTPException):
        pass
    except Exception as e:
        traceback.print_exc()
        raise HTTPException(
            status_code=500,
            detail="Ruh Roh, something has gone terribly wrong - Denied by eHelply."
        )

    if spend_maxed and exception_if_spend_maxed:
        raise HTTPException(
            status_code=400,
            detail="Project spend is maxed - Denied by eHelply. Increase max spend to complete this request."
        )
    elif spend_maxed:
        return False

    if project_not_enabled and exception_if_project_not_enabled:
        raise HTTPException(
            status_code=400,
            detail="Project is not enabled - Denied by eHelply. The project may be awaiting approval or disabled."
        )
    elif project_not_enabled:
        return False

    if exception_if_unauthorized:
        raise HTTPException(status_code=403, detail="Unauthorized - Denied by eHelply")
    else:
        return False

    #
    # return AuthRule(
    #     auth,
    #     AuthRule(auth).participant_has_node_on_target(
    #         node=node,
    #         target_identifier=service_target,
    #         partition="ehelply-resources"
    #     ),
    #     AuthRule(auth).participant_has_node_on_target(
    #         node=node,
    #         target_identifier=auth.project_uuid,
    #         partition="ehelply-cloud"
    #     ).customentity_has_node_on_target(
    #         node=node,
    #         target_identifier=service_target,
    #         partition="ehelply-cloud",
    #         entity_identifier=auth.project_uuid
    #     ),
    #     exception_if_unauthorized=exception_if_unauthorized
    # ).verify()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ehelply_microservice_library',
 'ehelply_microservice_library.cli',
 'ehelply_microservice_library.integrations',
 'ehelply_microservice_library.realtime',
 'ehelply_microservice_library.routers',
 'ehelply_microservice_library.utils',
 'ehelply_microservice_library.utils.constants']

package_data = \
{'': ['*']}

install_requires = \
['ehelply-bootstrapper>=0.13.0,<0.14.0',
 'ehelply-generator>=0.1.2,<0.2.0',
 'ehelply-python-sdk>=0.2.0,<0.3.0',
 'ehelply-updater>=0.1.3,<0.2.0',
 'jinja2>=2.11.3,<3.0.0']

entry_points = \
{'console_scripts': ['ehelply_microservice_library = '
                     'ehelply_microservice_library.cli.self_cli:cli_main']}

setup_kwargs = {
    'name': 'ehelply-microservice-library',
    'version': '1.3.8',
    'description': '',
    'long_description': '## Building\n* `poetry publish --build`\n\n## Development\n* `ehelply_microservice_library dev export-code-docs`\n* Clear cache: `poetry cache clear pypi --all`',
    'author': 'Shawn Clake',
    'author_email': 'shawn.clake@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://ehelply.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.9',
}


setup(**setup_kwargs)
